import {TablocOptions} from './options.js';

const LockState = {
  UNLOCKED: 0,
  BACKGROUND: 1,
  FOREGROUND: 2
};

class TablocBackGround {
  constructor() {
    this.locked = {};
    this.options = new TablocOptions();
  }

  isLocked(tabId) {
    return this.locked["locked_" + tabId];
  }

  async setLocked(tabId, value) {
    if (value) {
      this.locked["locked_" + tabId] = value;
      chrome.action.setTitle({
        title: chrome.i18n.getMessage("action_title_unlock"),
        tabId: tabId
      });
      chrome.action.setIcon({
        path: await this.options.get("ic_enabled"),
        tabId: tabId
      });
    } else {
      delete this.locked["locked_" + tabId];
      chrome.action.setTitle({
        title: chrome.i18n.getMessage("action_title"),
        tabId: tabId
      });
      chrome.action.setIcon({
        path: await this.options.get("ic_disabled"),
        tabId: tabId
      });
    }
  }

  async lockTab(tab, state) {
    const request = {
      command: "lockTab",
      ex_inpagelink: await this.options.get("ex_inpagelink"),
      ex_formsubmit: await this.options.get("ex_formsubmit"),
      ic_emoji: await this.options.get("ic_emoji")
    };
    const response = await chrome.tabs.sendMessage(tab.id, request);
    if (!response || response.result != "ok") {
      return;
    }
    this.setLocked(tab.id, state);

    await this.deleteFromAutoUrl(tab.url, LockState.UNLOCKED);
    await this.deleteFromAutoUrl(tab.url,
        state == LockState.FOREGROUND ? LockState.BACKGROUND : LockState.FOREGROUND);
    if (await this.getAutoLockState(tab.url) != state) {
      await this.addToAutoUrl(tab.url, state);
    }
  }

  async unlockTab(tab) {
    const request = {
      command: "unlockTab",
      ic_emoji: await this.options.get("ic_emoji")
    };
    const response = await chrome.tabs.sendMessage(tab.id, request);
    if (!response || response.result != "ok") {
      return;
    }
    this.setLocked(tab.id, LockState.UNLOCKED);

    await this.deleteFromAutoUrl(tab.url, LockState.BACKGROUND);
    await this.deleteFromAutoUrl(tab.url, LockState.FOREGROUND);
    if (await this.getAutoLockState(tab.url) != LockState.UNLOCKED) {
      await this.addToAutoUrl(tab.url, LockState.UNLOCKED);
    }
  }


  async addToAutoUrl(url, lockState) {
    const option = this.getOptionName(lockState);
    const current = await tabloc.options.get(option);
    await tabloc.options.set(option, this.escapeRegExp(url) + "\n" + current);
  }

  async deleteFromAutoUrl(url, lockState) {
    const escapedUrl = this.escapeRegExp(url);
    const option = this.getOptionName(lockState);
    const current = await tabloc.options.get(option);
    const updated =
        current.split("\n").filter(autoUrl => autoUrl != "" && autoUrl != escapedUrl).join("\n");
    if (updated != current) {
      await tabloc.options.set(option, updated);
    }
  }

  escapeRegExp(string) {
    return "^" + string.replace(/[.*+?^=!:${}()|[\]\/\\]/g, "\\$&") + "$";
  }

  getOptionName(lockState) {
    switch (lockState) {
      case LockState.BACKGROUND:
        return "auto_url_background";
      case LockState.FOREGROUND:
        return "auto_url_foreground";
      default:
        return "auto_url_exclude";
    }
  }

  async getAutoLockState(url) {
    const escapedUrl = this.escapeRegExp(url);
    const backGroundUrls = (await tabloc.options.get("auto_url_background")).split("\n");
    if (backGroundUrls.includes(escapedUrl)) {
      return LockState.BACKGROUND;
    }
    const foreGroundUrls = (await tabloc.options.get("auto_url_foreground")).split("\n");
    if (foreGroundUrls.includes(escapedUrl)) {
      return LockState.FOREGROUND;
    }
    for (const autoUrl of (await tabloc.options.get("auto_url_exclude")).split("\n")) {
      if (autoUrl && url.match(autoUrl)) {
        return LockState.UNLOCKED;
      }
    }
    for (const autoUrl of backGroundUrls) {
      if (autoUrl && url.match(autoUrl)) {
        return LockState.BACKGROUND;
      }
    }
    for (const autoUrl of foreGroundUrls) {
      if (autoUrl && url.match(autoUrl)) {
        return LockState.FOREGROUND;
      }
    }
    return LockState.UNLOCKED;
  }
};

const tabloc = new TablocBackGround();

chrome.action.onClicked.addListener(async (tab) => {
  if (!tabloc.isLocked(tab.id)) {
    const fg = await tabloc.options.get("fg_foreground")
    tabloc.lockTab(tab, fg ? LockState.FOREGROUND : LockState.BACKGROUND);
  } else {
    tabloc.unlockTab(tab);
  }
});

chrome.tabs.onUpdated.addListener(async (tabId, changeInfo, tab) => {
  if (changeInfo.status == "loading") {
    tabloc.setLocked(tabId, LockState.UNLOCKED);
    return;
  }
  if (changeInfo.status != "complete" || tabloc.isLocked(tabId)) {
    return;
  }
  const autoLockState = await tabloc.getAutoLockState(tab.url);
  if (autoLockState) {
    tabloc.lockTab(tab, autoLockState);
    return;
  }
  tabloc.setLocked(tabId, LockState.UNLOCKED);
});

chrome.runtime.onMessage.addListener(async (message, sender, sendResponse) => {
  if (message.command != "createTab") {
    sendResponse({});
    return;
  }

  let active;
  switch (tabloc.isLocked(sender.tab.id)) {
    case LockState.BACKGROUND:
      active = false;
      break;
    case LockState.FOREGROUND:
      active = true;
      break;
    default:
      active = await tabloc.options.get("fg_foreground");
  }
  
  let params = {
    url: message.url,
    active: active,
    openerTabId: sender.tab.id
  };
  if (await tabloc.options.get("pos_last")) {
    params.index = 1000000;
  } else if (await tabloc.options.get("pos_relative")) {
    params.index = sender.tab.index + 1;
  }
  try {
    chrome.tabs.create(params);
  } catch (e) {
    delete params.openerTabId;
    chrome.tabs.create(params);
  }
  sendResponse({result: "ok"});
});
